/*
 *  ======== TMP117.c ========
 *  TMP117 APIs for initialization and use of the TMP117 peripheral
 *
 *  DO NOT EDIT - This file is generated by the SysConfig tool for
 *  the TI Sensors in this application.
 */

#include <stddef.h>
#include <stdint.h>

#include "TMP117.h"
#include "mcu.h"

#define TMP117_0_CONFIG_VALUE (TMP117_CONFIG_CONV_MODE_CC | \
                               TMP117_CONFIG_CONV_CYCLE_1s | \
                               TMP117_CONFIG_AVG_MODE_8AVG | \
                               TMP117_CONFIG_TnA_ALERT | \
                               TMP117_CONFIG_ALERT_POL_LOW | \
                               TMP117_CONFIG_CRDY_EN_ALERT | \
                               TMP117_CONFIG_SOFT_RESET_DISABLE)
#define TMP117_0_THIGH_VALUE 0x6000U /* Alert High Limit is 192 Celsius */
#define TMP117_0_TLOW_VALUE 0x8000U /* Alert Low Limit is -256 Celsius */
#define TMP117_0_EEPROM_VALUE TMP117_EEPROM_EUN_DISABLE
#define TMP117_0_EEPROM1_VALUE 0x0000U
#define TMP117_0_EEPROM2_VALUE 0x0000U
#define TMP117_0_EEPROM3_VALUE 0x0000U
#define TMP117_0_OFFSET_TEMP_VALUE 0x0000U

static TMP117_State TMP117_0_state = {
    .config = TMP117_0_CONFIG_VALUE,

    .tHigh  = TMP117_0_THIGH_VALUE,
    .tLow   = TMP117_0_TLOW_VALUE,

    .offsetTemp = TMP117_0_OFFSET_TEMP_VALUE,

    .eEPROM1 = TMP117_0_EEPROM1_VALUE,
    .eEPROM2 = TMP117_0_EEPROM2_VALUE,
    .eEPROM3 = TMP117_0_EEPROM3_VALUE,
    /* Sensor's I2C bus ID and address */
    .busId   = 0,
    .devAddr = 0x48U,

    /* nonvolatile save mode for TMP117_0 registers */
    .nvmCommit = 0, /* don't save registers to EEPROM */

    /* preserve factory EEPROM NIST traceability data */
    .preserveNIST = 1, /* 1 => never overwrite NIST data, 0 => allow overwrite */

    /* one-shot delay time (ms) */
    .osWait  = 125    
};
const TMP117_Handle TMP117_0 = &TMP117_0_state;


#define MSB(u16) (((u16) & 0xFF00U) >> 8)
#define LSB(u16) ((u16) & 0xFFU)

/*
 *  ======== lock ========
 *  Lock out writes to EEPROM NVM
 */
static inline void lock(TMP117_Handle sensor)
{
    uint8_t txBuf[3];

    /* EEPROM_Unlock Register Write to Lock the NVM */
    txBuf[0] = TMP117_EEPROM;
    txBuf[1] = MSB(TMP117_EEPROM_EUN_DISABLE);
    txBuf[2] = LSB(TMP117_EEPROM_EUN_DISABLE);
    mcu_i2cTransfer(sensor->busId, sensor->devAddr, txBuf, 3, NULL, 0);
}

/*
 *  ======== unlock ========
 *  Unlock EEPROM NVM so sensor register writes will be saved
 */
static inline void unlock(TMP117_Handle sensor)
{
    uint8_t txBuf[3];

    /* EEPROM_Unlock Register Write to Unlock the NVM */
    txBuf[0] = TMP117_EEPROM;
    txBuf[1] = MSB(TMP117_EEPROM_EUN_ENABLE);
    txBuf[2] = LSB(TMP117_EEPROM_EUN_ENABLE);
    mcu_i2cTransfer(sensor->busId, sensor->devAddr, txBuf, 3, NULL, 0);
}

/*
 *  ======== pollBusy ========
 *  Poll EEPROM_Busy flag until it's 0 (not busy)
 */
static void pollBusy(TMP117_Handle sensor)
{
    uint8_t txBuf[1];
    uint8_t rxBuf[2];

    /* verify that EEPROM isn't busy */
    do {
        /* EEPROM_Unlock Register Read to get the Busy flag */
        txBuf[0] = TMP117_EEPROM;
        mcu_i2cTransfer(sensor->busId, sensor->devAddr, txBuf, 1, rxBuf, 2);

        /* EEPROM_Busy is in the MSB of the unlock register */
    } while (rxBuf[0] & MSB(TMP117_EEPROM_EEPROM_BUSY_ENABLE));
}

/*
 *  ======== writeDelay ========
 *  If necessary, wait for a register write to be saved to EEPROM
 */
static void writeDelay(TMP117_Handle sensor)
{
    if (!sensor->nvmCommit) {
        return; /* nothing to do if we're not saving to EEPROM */
    }

    /* Wait delay 7 milliseconds for EEPROM to not be busy */
    mcu_msWait(7);

    /* wait for sensor to not be busy */
    pollBusy(sensor);
}

/*
 *  ======== writeReg ========
 */
static void writeReg(TMP117_Handle sensor, int regAddr, uint16_t value)
{
    uint8_t txBuf[3];

    txBuf[0] = regAddr;
    txBuf[1] = MSB(value);
    txBuf[2] = LSB(value);
    mcu_i2cTransfer(sensor->busId, sensor->devAddr, txBuf, 3, NULL, 0);
    writeDelay(sensor);
}

/*
 *  ======== reset ========
 *  Reset the sensor and wait for it to be ready
 */
static void reset(TMP117_Handle sensor)
{
    uint8_t txBuf[3];

    txBuf[0] = TMP117_CONFIG;
    txBuf[1] = MSB(TMP117_CONFIG_SOFT_RESET_ENABLE);
    txBuf[2] = LSB(TMP117_CONFIG_SOFT_RESET_ENABLE);
    mcu_i2cTransfer(sensor->busId, sensor->devAddr, txBuf, 3, NULL, 0);
    mcu_msWait(2);
}

/*
 *  ======== TMP117_config ========
 *  Configure device with current settings
 *
 *  Note: after a sensor reset, the EEPROM is locked and remains locked for
 *  approximatly 1.5 ms and all register writes are ignored.
 */
void TMP117_config(TMP117_Handle sensor)
{
    mcu_i2cInit(sensor->busId);

    /* Ensure sensor is not processing a reset */
    pollBusy(sensor);

    if (sensor->nvmCommit) {
        /* Unlock EEPROM if we're saving registers */
        unlock(sensor);
    }

    /* High Limit Register Write */
    writeReg(sensor, TMP117_THIGH, sensor->tHigh);

    /* Low Limit Register Write */
    writeReg(sensor, TMP117_TLOW, sensor->tLow);

    /* Temperature Offset Register Write */
    writeReg(sensor, TMP117_OFFSET_TEMP, sensor->offsetTemp);
    
    if (!sensor->preserveNIST) {
        /* EEPROMx Registers Write */
        writeReg(sensor, TMP117_EEPROM1, sensor->eEPROM1);
        writeReg(sensor, TMP117_EEPROM2, sensor->eEPROM2);
        writeReg(sensor, TMP117_EEPROM3, sensor->eEPROM3);
    }

    /* Configuration Register Write */
    /* Ensure SOFT_RESET isn't set - otherwise CONFIG setting won't work */
    writeReg(sensor, TMP117_CONFIG, sensor->config & (~TMP117_CONFIG_SOFT_RESET_ENABLE));

    /* lock EEPROM to ensure it's no longer writeable */
    if (sensor->config & TMP117_CONFIG_SOFT_RESET_ENABLE) {
        reset(sensor); /* reset also locks EEPROM */
    }
    else {
        lock(sensor);
    }
}

/*
 *  ======== TMP117_read ========
 *  Read raw temperature value
 */
int32_t TMP117_read(TMP117_Handle sensor)
{
    uint8_t txBuf[3];
    uint8_t rxBuf[2];
    int32_t tmp;

    /* If needed, trigger one shot measurement */
    if (sensor->config & TMP117_CONFIG_CONV_MODE_SD) {
        /* Reset one-shot enable */
        txBuf[0] = TMP117_CONFIG;
        txBuf[1] = MSB(sensor->config | TMP117_OS_ENABLE);
        txBuf[2] = LSB(sensor->config);
        mcu_i2cTransfer(sensor->busId, sensor->devAddr, txBuf, 3, NULL, 0);

        /* Wait for conversion to complete */
        mcu_msWait(sensor->osWait);
    }

    /* Temperature Register Read */
    txBuf[0] = TMP117_TEMP;
    mcu_i2cTransfer(sensor->busId, sensor->devAddr, txBuf, 1, rxBuf, 2);

    /* Sign extend and combine */
    tmp = (((int32_t)(int8_t)rxBuf[0]) << 8) | rxBuf[1];

    return (tmp);
}

/*
 *  ======== TMP117_toIntCelsius ========
 *  Convert raw temperature register value to degrees Celsius rounded to the
 *  nearest integer
 */
int32_t TMP117_toIntCelsius(int32_t x)
{
    /* Optional: Add bias to round before the final truncation */
    x += (x >= 0) ? (1 << 6) : -(1 << 6);

    /* Convert Q7 value to whole number */
    x /= 1 << 7; /* use division so small negative values round to 0 */

    return x;
}

/*
 *  ======== TMP117_toMilliCelsius ========
 *  Convert raw temperature register value to milli-degrees Celsius
 */
int32_t TMP117_toMilliCelsius(int32_t x)
{
    /* Scale to milli-degrees, convert Q7 value to a whole number */
    return ((1000 * x) >> 7);
}

/*
 *  ======== TMP117_toFloatCelsius ========
 *  Convert raw temperature register value to degrees Celsius
 */
float TMP117_toFloatCelsius(int32_t x)
{
    /* Convert Q7 value to a float */
    return ((float)x * 0.0078125f);
}


